#include "config.h"

#include <stdlib.h>
#include <iostream>
#include <fstream>
#include <string>
#include <cstring>
#include <ctime>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <fcntl.h>
#include <errno.h>

#include <FL/Fl.H>
#include <FL/Enumerations.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Button.H>
#include <FL/Fl_Group.H>
#include <FL/Fl_Sys_Menu_Bar.H>
#include <FL/x.H>
#include <FL/Fl_Help_Dialog.H>
#include <FL/Fl_Menu_Item.H>

#ifdef __WIN32__
#	include "compat.h"
#	define dirent fl_dirent_no_thanks
#	include <dirent.h>
#endif

#include <FL/filename.H>

#include <FL/x.H>

#include "flclient.h"
#include "dialogs.h"
#include "status.h"
//#include "debug.h"
#include "util.h"
#include "fileselect.h"
#include "icons.h"
#include "XmlRpcClient.h"

Fl_Double_Window *mainwindow;

void about()
{
	std::string msg = "\
copyright W1HKJ, 2021\n\
w1hkj@@w1hkj.com";
	fl_message2("flclient %s\n\n%s", VERSION, msg.c_str());
}

#if defined(__WIN32__) && defined(PTW32_STATIC_LIB)
static void ptw32_cleanup(void)
{
	(void)pthread_win32_process_detach_np();
}

void ptw32_init(void)
{
	(void)pthread_win32_process_attach_np();
	atexit(ptw32_cleanup);
}
#endif // __WIN32__

void exit_main(Fl_Widget *w)
{
	if (Fl::event_key() == FL_Escape)
		return;
	client_stop_thread();
	mainwindow->hide();
}

void cb_Exit()
{
	client_stop_thread();
	mainwindow->hide();
}

int parse_args(int argc, char **argv, int& idx)
{
	if (strcasecmp("--help", argv[idx]) == 0) {
		printf("Usage: \n\
  --help this help text\n\
  --version\n\
    addr:port [localhost:12345]\n\n");
		exit(0);
	}
	if (strcasecmp("--version", argv[idx]) == 0) {
		printf("Version: " VERSION "\n");
		exit (0);
	}
	std::string strarg = argv[idx];
	size_t p = strarg.find(":");
	if (p != std::string::npos) {
		progStatus.address = strarg.substr(0, p);
		progStatus.port = strarg.substr(p+1);
	}
	return 0;
}

static std::string pname = "";

int main (int argc, char *argv[])
{
	pname = argv[0];
	size_t pn = pname.rfind("/");
	if (pn != std::string::npos) pname.erase(0, pn + 1);
	pn = pname.rfind("\\");
	if (pn != std::string::npos) pname.erase(0, pn + 1);
	XmlRpc::set_pname(pname);

	int arg_idx;
	Fl::args(argc, argv, arg_idx, parse_args);

	mainwindow = client_dialog();
	mainwindow->callback(exit_main);
	init_commands();

	Fl::lock();

#if defined(__WIN32__) && defined(PTW32_STATIC_LIB)
	ptw32_init();
#endif

	client_start_thread();

	mainwindow->show();

	return Fl::run();
}
